# Example of how to add a missing code to an existing DBHMEasurement in  a CTFS database
# Shameema Esufali revised June 2020

-- **********************************************************************************************
-- Set up the database.  Replace the test database "blue" with the last frozen 
-- version of your database. Eg USE current_yoursite;  SOURCE ../bci_current_20August2020.sql;

DROP DATABASE IF EXISTS blue;
CREATE DATABASE IF NOT EXISTS blue;
USE blue;
SOURCE ../sampledatabases/blue.sql;


-- You must run the code below in your database.  Once you have run it for one script it is 
-- not necessary to run it for others but it will not cause a problem if you do. 
-- Do not remove the commands below. Change the path to reflect your directory path.
# 1. Installs a procedure that creates DBHAttributes from codes separated by a semicolon.
# The following columns must exist in the table tempinserttrees. 
# codes varchar(20),
# dbhid int,
# 2. Installs Log Procedures. 
SOURCE Procedures/LogProcedures.sql;
SOURCE Procedures/Utilities.sql;

# Some older databases have the old and new columns as NOT NULL
# the statement below corrects the error. 
ALTER TABLE Log MODIFY old VARCHAR(2048), MODIFY new VARCHAR(2048);
SELECT @n1:=count(*) FROM Log;

# This table is required in the database. 
DROP TABLE IF EXISTS TempLogFeeder;
CREATE TABLE TempLogFeeder(
  ChangedTableID int(10),
  ChangedColumn varchar(64),
  ChangeReason VARCHAR(256)
  );
  
-- end database setup **********************************************************************  
# *******************************************************************************************************
# 1.  In this example I update a code to an existing stem in Census 4.  To identify the code 
# I need to identify the measurement for the stem for the census in which the code needs to be changed. 
# i.e. I need the DBH identifier for the measurement that needs to be coded or recoded. 
# Notice the syntax for an assignment of a value is :=
# *******************************************************************************************************
SET @tag="000040", @stemtag=1, @plotcensus=4, @oldcode="-1"; 

# @newcode must be set to a valid tsmcode from the TSMAttributes table. 
# Note: spelling and capitalization must be exactly the same as in the table. 

SET @newcode="D";

SELECT @newtsmid := tsmid FROM TSMAttributes WHERE tsmcode=@newcode;
SELECT @oldtsmid :=tsmid FROM TSMAttributes WHERE tsmcode=@oldcode;

# I put these select statements into programs to show me the variables
SELECT @tag,@stemtag,@dbhid,@oldcode,@newcode,@oldtsmid,@newtsmid,@dbhattid;

# Get the DBHID for that census for the stem in question.  First get the censusid;

SELECT @censusid:= censusid FROM Census WHERE plotcensusnumber=@plotcensus;

SELECT @dbhid:=c.dbhid, @dbhattid:=DBHAttID FROM Tree a, Stem b, DBH c, DBHAttributes d
  WHERE a.treeid=b.treeid
    AND b.stemid=c.stemid
    AND c.dbhid=d.dbhid
    AND c.censusid=@censusid
    AND a.tag=@tag 
    AND b.stemtag=@stemtag
    AND d.tsmid=@oldtsmid;

# Checking the variables are all properly filled
SELECT @tag,@stemtag,@dbhid,@oldcode,@newcode,@oldtsmid,@newtsmid,@dbhattid; 

  
# *******************************************************************************************************
# ***** CREATE LOG RECORDS FOR DBHattributes before UPDATE
#  Note: You should be able to use this code without any changes except the description or reason .
# *******************************************************************************************************

SET @personnelid=1, @action="U",@changereason="found evidence of tree death" ;

TRUNCATE TempLogFeeder;
INSERT INTO TempLogFeeder (ChangedTableID,ChangedColumn,ChangeReason) 
  SELECT @dbhattid,"tsmid",@changereason
  FROM DBHAttributes  WHERE DBHAttID =@dbhattid;

CALL logRecordSet(@personnelid,"DBHAttributes",@action); 
    
UPDATE DBHAttributes SET tsmid =@newtsmid 
  WHERE dbhid=@dbhid 
    AND tsmid=@oldtsmid;



# *******************************************************************************************************
# 2.  In this example I am simply adding a code to an existing measurement.  The code may
# have been missed in data entry and upload. 
# *******************************************************************************************************
SET @tag="000044", @stemtag=1, @plotcensus=4; 

# @code must be set to a valid tsmcode from the TSMAttributes table. 
# Note spelling and capitalization must be exactly the same as in the table. 

SET @code="LL";
SELECT @tsmid := tsmid FROM TSMAttributes WHERE tsmcode=@code;

# Get the DBHID for that census for the stem in question.  First get the censusid;
SELECT @censusid:= censusid FROM Census WHERE plotcensusnumber=@plotcensus;

SELECT @dbhid:=c.dbhid FROM Tree a, Stem b, DBH c
  WHERE a.treeid=b.treeid
    AND b.stemid=c.stemid
    AND c.censusid=@censusid
    AND a.tag=@tag 
    AND b.stemtag=@stemtag;
  
INSERT INTO DBHAttributes (tsmid,dbhid) VALUES(@tsmid,@dbhid);

# *******************************************************************************************************
# ***** CREATE LOG RECORDS FOR DBHattributes
#  Note: You should be able to use this code without any changes except the description or reason .
# *******************************************************************************************************

SET @personnelid=1, @action="I", @changereason="missed leaning code" ;

TRUNCATE TempLogFeeder;
INSERT INTO TempLogFeeder (ChangedTableID,ChangedColumn,ChangeReason) 
  SELECT dbhattid,"tsmid",@changereason
  FROM DBHAttributes  WHERE DBHID =@dbhid and tsmid=@tsmid;

CALL logRecordSet(@personnelid,"DBHAttributes",@action); 


# *******************************************************************************************************
# Rebuild Views. Replace the first parameter with your database name 
# *******************************************************************************************************

CALL ctfsweb_webuser.CreateViewTaxonomy("blue","'foo'");
CALL ctfsweb_webuser.CreateFullView( "blue","ViewFullTable");


# *******************************************************************************************************
# 3.  In this example I am deleting a code from an existing measurement.  This example deletes
#     the code that I added in the previous example.   NOTE:  The delete needs to be logged BEFORE 
#     record is deleted from the database. 
# *******************************************************************************************************
SET @tag="000044", @stemtag=1, @plotcensus=4; 

# @code must be set to a valid tsmcode from the TSMAttributes table. 
# Note spelling and capitalization must be exactly the same as in the table. 

SET @code="LL";
SELECT @tsmid := tsmid FROM TSMAttributes WHERE tsmcode=@code;

# Get the DBHID for that census for the stem in question.  First get the censusid;
SELECT @censusid:= censusid FROM Census WHERE plotcensusnumber=@plotcensus;

SELECT @dbhid:=c.dbhid FROM Tree a, Stem b, DBH c
  WHERE a.treeid=b.treeid
    AND b.stemid=c.stemid
    AND c.censusid=@censusid
    AND a.tag=@tag 
    AND b.stemtag=@stemtag;
    
# *******************************************************************************************************
# ***** CREATE LOG RECORDS FOR DBHattributes
#  Note: You should be able to use this code without any changes except the description or reason .
#  The record is logged and THEN deleted. 
# *******************************************************************************************************

SET @personnelid=1, @action="D";

SELECT "Logging delete";
TRUNCATE TempLogFeeder;
INSERT INTO TempLogFeeder (ChangedTableID,ChangedColumn,ChangeReason) 
  SELECT dbhattid,"tsmid","removing leaning code added in the insert example" 
  FROM DBHAttributes  WHERE DBHID =@dbhid and tsmid=@tsmid;

CALL logRecordSet(@personnelid,"DBHAttributes",@action); 


# *******************************************************************************************************
SELECT "Deleting record";
# *******************************************************************************************************

DELETE FROM DBHAttributes WHERE dbhid=@dbhid AND tsmid = @tsmid;




# *******************************************************************************************************
# Rebuild Views. Replace the first parameter with your database name 
# *******************************************************************************************************

CALL ctfsweb_webuser.CreateViewTaxonomy("blue","'foo'");
CALL ctfsweb_webuser.CreateFullView( "blue","ViewFullTable");


