/*
These are examples of scripts for taxonomic revisions to change species names in the Species table.
ChangeCodeID refers to those in the TreeTaxChange table.
Following are several possible scenarios.
*/
SET @PersonnelID=1;    # REPLACE WITH PersonnelID of person responsible for this Taxonomic change.


-- I. Species A is obsolete and all of SpeciesA changes to SpeciesB where GenusA=GenusB and SpeciesNameA<>SpeciesnameB And SpeciesB already existed in Species table --

SELECT @oldSpID:=SpeciesID FROM Species WHERE CurrentTaxonFlag=1 AND mnemonic='oldMnemonic'; # REPLACE WITH Species code of Species to be corrected
SELECT @newSpID:=SpeciesID FROM Species WHERE CurrentTaxonFlag=1 AND mnemonic='newMnemonic'; # REPLACE WITH Species code of correct Species

-- Insert record into CurrentObsolete table indicating change 
INSERT INTO CurrentObsolete (SpeciesID,ObsoleteSpeciesID,ChangeDate,ChangeCodeID) VALUES (@newSpID,@oldSpID,NOW(),'14');  
-- Change the flags in the Species table for the obsolete species 
UPDATE Species SET CurrentTaxonFlag=0,ObsoleteTaxonFlag=1 WHERE SpeciesID=@oldSpID;
-- Log the changes from the Tree table
INSERT INTO Log (PersonnelID,ChangedTable,PrimaryKey,ChangedColumn,ChangeDate,ChangeTime,Action,Old,New) 
   SELECT @PersonnelID,'Tree',TreeID,'SpeciesID',CURDATE(),NOW(),'U',@oldSpID,@newSpID FROM Tree WHERE SpeciesID=@oldSpID;
-- Update the Species in the Tree table   
UPDATE Tree SET SpeciesID=@newSpID WHERE SpeciesID=@oldSpID;   


-- II. Species A is obsolete and all of SpeciesA changes to SpeciesB where GenusA<>GenusB and SpeciesnameA<>SpeciesnameB And SpeciesB already exists in Species table. --
-- NOTE: Same as above except that the ChangeCodeID changes from '14' to '13'

SELECT @oldSpID:=SpeciesID FROM Species WHERE CurrentTaxonFlag=1 AND mnemonic='oldMnemonic'; # REPLACE WITH Species code of Species to be corrected
SELECT @newSpID:=SpeciesID FROM Species WHERE CurrentTaxonFlag=1 AND mnemonic='newMnemonic'; # REPLACE WITH Species code of correct Species

-- Insert record into CurrentObsolete table indicating change 
INSERT INTO CurrentObsolete (SpeciesID,ObsoleteSpeciesID,ChangeDate,ChangeCodeID) VALUES (@newSpID,@oldSpID,NOW(),'13');  
-- Change the flags in the Species table for the obsolete species 
UPDATE Species SET CurrentTaxonFlag=0,ObsoleteTaxonFlag=1 WHERE SpeciesID=@oldSpID;
-- Log the changes from the Tree table
INSERT INTO Log (PersonnelID,ChangedTable,PrimaryKey,ChangedColumn,ChangeDate,ChangeTime,Action,Old,New) 
   SELECT @PersonnelID,'Tree',TreeID,'SpeciesID',CURDATE(),NOW(),'U',@oldSpID,@newSpID FROM Tree WHERE SpeciesID=@oldSpID;
-- Update the Species in the Tree table   
UPDATE Tree SET SpeciesID=@newSpID WHERE SpeciesID=@oldSpID;   


-- III. Species A is obsolete and all trees in SpeciesA change to SpeciesB where GenusA=GenusB and SpeciesnameA<>SpeciesnameB And SpeciesB is new to Species table.

SELECT @genusid:=genusid FROM Genus WHERE genus= "Unidentified"; # REPLACE WITH GENUS OF NEW SPECIES
SET @idlevel="multiple";            # REPLACE WITH IDLEVEL OF NEW SPECIES (Possible IDLEVELs: species,genus,family,none,subspecies,multiple)
SET @SpName:='sp. 3';               # REPLACE WITH NEW SPECIES NAME
SET @Auth:='Authority of Species';  # REPLACE WITH AUTHORITY OF THE NEW SPECIES
SET @newSpCode:='newSpcode';        # REPLACE WITH SPECIES CODE OF THE NEW SPECIES

SELECT @oldSpID:=SpeciesID FROM Species WHERE CurrentTaxonFlag=1 AND mnemonic='oldMnemonic'; # REPLACE WITH Species code of Species to be corrected
SELECT @newSpID:=MAX(SpeciesID)+1 FROM Species;

-- Insert new species information into the Species table
INSERT INTO Species (SpeciesID,CurrentTaxonFlag,ObsoleteTaxonFlag,GenusID,SpeciesName,Mnemonic,Authority,IDLevel) VALUES (@newSpID,1,0,@genusid,@SpName,@newSpCode,@Auth,@idlevel);

-- Insert record into CurrentObsolete table indicating change 
INSERT INTO CurrentObsolete (SpeciesID,ObsoleteSpeciesID,ChangeDate,ChangeCodeID) VALUES (@newSpID,@oldSpID,NOW(),'11');  # Notice ChangeCodeID is now '11'
-- Change the flags in the Species table for the obsolete species 
UPDATE Species SET CurrentTaxonFlag=0,ObsoleteTaxonFlag=1 WHERE SpeciesID=@oldSpID;
-- Log the changes from the Tree table
INSERT INTO Log (PersonnelID,ChangedTable,PrimaryKey,ChangedColumn,ChangeDate,ChangeTime,Action,Old,New) 
   SELECT @PersonnelID,'Tree',TreeID,'SpeciesID',CURDATE(),NOW(),'U',@oldSpID,@newSpID FROM Tree WHERE SpeciesID=@oldSpID;
-- Update the Species in the Tree table   
UPDATE Tree SET SpeciesID=@newSpID WHERE SpeciesID=@oldSpID;   


-- IV. Species A is obsolete and SpeciesA changes to SpeciesB where GenusA<>GenusB and SpeciesnameA<>SpeciesnameB And SpeciesB is new to Species table.
-- NOTE: Same as III above except that the ChangeCodeID changes from '11' to '12'
 
SELECT @genusid:=genusid FROM Genus WHERE genus= "Unidentified"; # REPLACE WITH GENUS OF NEW SPECIES
SET @idlevel="multiple";            # REPLACE WITH IDLEVEL OF NEW SPECIES (Possible IDLEVELs: species,genus,family,none,subspecies,multiple)
SET @SpName:='sp. 3';               # REPLACE WITH NEW SPECIES NAME
SET @Auth:='Authority of Species';  # REPLACE WITH AUTHORITY OF THE NEW SPECIES
SET @newSpCode:='newSpcode';        # REPLACE WITH SPECIES CODE OF THE NEW SPECIES

SELECT @oldSpID:=SpeciesID FROM Species WHERE CurrentTaxonFlag=1 AND mnemonic='oldMnemonic'; # REPLACE WITH Species code of Species to be corrected
SELECT @newSpID:=MAX(SpeciesID)+1 FROM Species;

-- Insert new species information into the Species table
INSERT INTO Species (SpeciesID,CurrentTaxonFlag,ObsoleteTaxonFlag,GenusID,SpeciesName,Mnemonic,Authority,IDLevel) VALUES (@newSpID,1,0,@genusid,@SpName,@newSpCode,@Auth,@idlevel);

-- Insert record into CurrentObsolete table indicating change 
INSERT INTO CurrentObsolete (SpeciesID,ObsoleteSpeciesID,ChangeDate,ChangeCodeID) VALUES (@newSpID,@oldSpID,NOW(),'12');  # Notice ChangeCodeID is now '12'
-- Change the flags in the Species table for the obsolete species 
UPDATE Species SET CurrentTaxonFlag=0,ObsoleteTaxonFlag=1 WHERE SpeciesID=@oldSpID;
-- Log the changes from the Tree table
INSERT INTO Log (PersonnelID,ChangedTable,PrimaryKey,ChangedColumn,ChangeDate,ChangeTime,Action,Old,New) 
   SELECT @PersonnelID,'Tree',TreeID,'SpeciesID',CURDATE(),NOW(),'U',@oldSpID,@newSpID FROM Tree WHERE SpeciesID=@oldSpID;
-- Update the Species in the Tree table   
UPDATE Tree SET SpeciesID=@newSpID WHERE SpeciesID=@oldSpID;   






